@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -138,6 +138,71 @@ select_theme (GSManager *manager)
         return theme;
 }
 
+
+static GSJob *
+lookup_job_for_window (GSManager *manager,
+                       GSWindow  *window)
+{
+        GSJob *job;
+
+        if (manager->priv->jobs == NULL) {
+                return NULL;
+        }
+
+        job = g_hash_table_lookup (manager->priv->jobs, window);
+
+        return job;
+}
+
+static void
+manager_maybe_stop_job_for_window (GSManager *manager,
+                                   GSWindow  *window)
+{
+        GSJob *job;
+
+        job = lookup_job_for_window (manager, window);
+
+        if (job == NULL) {
+                gs_debug ("Job not found for window");
+                return;
+        }
+
+        gs_job_stop (job);
+}
+
+static void
+manager_maybe_start_job_for_window (GSManager *manager,
+                                    GSWindow  *window)
+{
+        GSJob *job;
+
+        job = lookup_job_for_window (manager, window);
+
+        if (job == NULL) {
+                gs_debug ("Job not found for window");
+                return;
+        }
+
+        if (! manager->priv->dialog_up) {
+                if (! manager->priv->throttle_enabled) {
+                        if (! gs_job_is_running (job)) {
+                                if (! gs_window_is_obscured (window)) {
+                                        gs_debug ("Starting job for window");
+                                        gs_job_start (job);
+                                } else {
+                                        gs_debug ("Window is obscured deferring start of job");
+                                }
+                        } else {
+                                gs_debug ("Not starting job because job is running");
+                        }
+                } else {
+                        gs_debug ("Not starting job because throttled");
+                }
+        } else {
+                gs_debug ("Not starting job because dialog is up");
+        }
+}
+
 static void
 cycle_job (GSWindow  *window,
            GSJob     *job,
@@ -149,7 +214,7 @@ cycle_job (GSWindow  *window,
 
         gs_job_stop (job);
         gs_job_set_theme (job, theme, NULL);
-        gs_job_start (job);
+        manager_maybe_start_job_for_window (manager, window);
 }
 
 static void
@@ -168,7 +233,7 @@ throttle_job (GSWindow  *window,
         if (manager->priv->throttle_enabled) {
                 gs_job_stop (job);
         } else {
-                gs_job_start (job);
+                manager_maybe_start_job_for_window (manager, window);
         }
 }
 
@@ -188,7 +253,7 @@ resume_job (GSWindow  *window,
         if (gs_job_is_running (job)) {
                 gs_job_suspend (job, FALSE);
         } else {
-                gs_job_start (job);
+                manager_maybe_start_job_for_window (manager, window);
         }
 }
 
@@ -276,6 +341,19 @@ gs_manager_set_throttle_enabled (GSManager *manager,
         }
 }
 
+void
+gs_manager_get_lock_active (GSManager *manager,
+                            gboolean  *lock_active)
+{
+        if (lock_active != NULL) {
+                *lock_active = FALSE;
+        }
+
+        g_return_if_fail (GS_IS_MANAGER (manager));
+
+        *lock_active = manager->priv->lock_active;
+}
+
 void
 gs_manager_set_lock_active (GSManager *manager,
                             gboolean   lock_active)
@@ -431,6 +509,8 @@ gs_manager_cycle (GSManager *manager)
         g_return_val_if_fail (manager != NULL, FALSE);
         g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
 
+        gs_debug ("cycling jobs");
+
         if (! manager->priv->active) {
                 return FALSE;
         }
@@ -783,6 +863,12 @@ window_dialog_up_cb (GSWindow  *window,
         gs_debug ("Handling dialog up");
 
         manager->priv->dialog_up = TRUE;
+        /* Make all other windows insensitive so we don't get events */
+        for (l = manager->priv->windows; l; l = l->next) {
+                if (l->data != window) {
+                        gtk_widget_set_sensitive (GTK_WIDGET (l->data), FALSE);
+                }
+        }
 
         /* Move keyboard and mouse grabs so dialog can be used */
         gs_grab_window (gs_window_get_gdk_window (window),
@@ -793,13 +879,6 @@ window_dialog_up_cb (GSWindow  *window,
            the dialog can be used.  We'll regrab it when the dialog goes down. */
         gs_grab_release_mouse ();
 
-        /* Make all other windows insensitive so we don't get events */
-        for (l = manager->priv->windows; l; l = l->next) {
-                if (l->data != window) {
-                        gtk_widget_set_sensitive (GTK_WIDGET (l->data), FALSE);
-                }
-        }
-
         if (! manager->priv->throttle_enabled) {
                 gs_debug ("Suspending jobs");
 
@@ -826,38 +905,48 @@ window_dialog_down_cb (GSWindow  *window,
                 gtk_widget_set_sensitive (GTK_WIDGET (l->data), TRUE);
         }
 
+        manager->priv->dialog_up = FALSE;
+
         if (! manager->priv->throttle_enabled) {
                 manager_resume_jobs (manager);
         }
-
-        manager->priv->dialog_up = FALSE;
 }
 
-static void
-manager_maybe_start_job_for_window (GSManager *manager,
-                                    GSWindow  *window)
+static GSWindow *
+find_window_at_pointer (GSManager *manager)
 {
-        GSJob *job;
+        GdkDisplay *display;
+        GdkScreen  *screen;
+        int         monitor;
+        int         x, y;
+        GSWindow   *window;
+        int         screen_num;
+        GSList     *l;
 
-        if (manager->priv->jobs == NULL) {
-                return;
-        }
+        display = gdk_display_get_default ();
+        gdk_display_get_pointer (display, &screen, &x, &y, NULL);
+        monitor = gdk_screen_get_monitor_at_point (screen, x, y);
+        screen_num = gdk_screen_get_number (screen);
 
-        job = g_hash_table_lookup (manager->priv->jobs, window);
-
-        if (job == NULL) {
-                gs_debug ("Job not found for window");
-                return;
-        }
-
-        if (! manager->priv->dialog_up) {
-                if (! manager->priv->throttle_enabled) {
-                        if (! gs_job_is_running (job)) {
-                                gs_debug ("Starting job for window");
-                                gs_job_start (job);
-                        }
+        /* Find the gs-window that is on that screen */
+        window = NULL;
+        for (l = manager->priv->windows; l; l = l->next) {
+                GSWindow *win = GS_WINDOW (l->data);
+                if (gs_window_get_screen (win) == screen
+                    && gs_window_get_monitor (win) == monitor) {
+                        window = win;
                 }
         }
+
+        if (window == NULL) {
+                gs_debug ("WARNING: Could not find the GSWindow for screen %d", screen_num);
+                /* take the first one */
+                window = manager->priv->windows->data;
+        } else {
+                gs_debug ("Requesting unlock for screen %d", screen_num);
+        }
+
+        return window;
 }
 
 static gboolean
@@ -1000,16 +1089,58 @@ window_show_cb (GSWindow  *window,
         manager_show_window (manager, window);
 }
 
+static gboolean
+window_activity_cb (GSWindow  *window,
+                    GSManager *manager)
+{
+        gboolean handled;
+
+        handled = gs_manager_request_unlock (manager);
+
+        return handled;
+}
+
+static void
+maybe_set_window_throttle (GSManager *manager,
+                           GSWindow  *window,
+                           gboolean   throttled)
+{
+        if (throttled) {
+                manager_maybe_stop_job_for_window (manager, window);
+        } else {
+                manager_maybe_start_job_for_window (manager, window);
+        }
+}
+
+static void
+window_obscured_cb (GSWindow   *window,
+                    GParamSpec *pspec,
+                    GSManager  *manager)
+{
+        gboolean obscured;
+
+        obscured = gs_window_is_obscured (window);
+        gs_debug ("Handling window obscured: %s", obscured ? "obscured" : "unobscured");
+
+        maybe_set_window_throttle (manager, window, obscured);
+
+        if (! obscured) {
+                gs_manager_request_unlock (manager);
+        }
+}
+
 static void
 disconnect_window_signals (GSManager *manager,
                            GSWindow  *window)
 {
         g_signal_handlers_disconnect_by_func (window, window_deactivated_cb, manager);
+        g_signal_handlers_disconnect_by_func (window, window_activity_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_dialog_up_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_dialog_down_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_show_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_map_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_map_event_cb, manager);
+        g_signal_handlers_disconnect_by_func (window, window_obscured_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_unmap_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_grab_broken_cb, manager);
 }
@@ -1027,6 +1158,8 @@ connect_window_signals (GSManager *manager,
 {
         g_signal_connect_object (window, "destroy",
                                  G_CALLBACK (window_destroyed_cb), manager, 0);
+        g_signal_connect_object (window, "activity",
+                                 G_CALLBACK (window_activity_cb), manager, 0);
         g_signal_connect_object (window, "deactivated",
                                  G_CALLBACK (window_deactivated_cb), manager, 0);
         g_signal_connect_object (window, "dialog-up",
@@ -1039,6 +1172,8 @@ connect_window_signals (GSManager *manager,
                                  G_CALLBACK (window_map_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "map_event",
                                  G_CALLBACK (window_map_event_cb), manager, G_CONNECT_AFTER);
+        g_signal_connect_object (window, "notify::obscured",
+                                 G_CALLBACK (window_obscured_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "unmap",
                                  G_CALLBACK (window_unmap_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "grab_broken_event",
@@ -1187,6 +1322,10 @@ gs_manager_activate (GSManager *manager)
                                FADE_TIMEOUT,
                                (GSFadeDoneFunc)fade_done_cb,
                                manager);
+
+                while (manager->priv->fading) {
+                        gtk_main_iteration ();
+                }
         } else {
                 show_windows (manager->priv->windows);
         }
@@ -1222,7 +1361,6 @@ gs_manager_deactivate (GSManager *manager)
         manager->priv->active = FALSE;
         manager->priv->activate_time = 0;
         manager->priv->lock_active = FALSE;
-        manager->priv->throttle_enabled = FALSE;
 
         return TRUE;
 }
@@ -1251,20 +1389,22 @@ gs_manager_get_active (GSManager *manager)
         return manager->priv->active;
 }
 
-void
+gboolean
 gs_manager_request_unlock (GSManager *manager)
 {
-        GSList     *l;
-        GdkDisplay *display;
-        GdkScreen  *screen;
-        int         screen_num;
+        GSWindow *window;
 
-        g_return_if_fail (manager != NULL);
-        g_return_if_fail (GS_IS_MANAGER (manager));
+        g_return_val_if_fail (manager != NULL, FALSE);
+        g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
 
         if (! manager->priv->active) {
                 gs_debug ("Request unlock but manager is not active");
-                return;
+                return FALSE;
+        }
+
+        if (manager->priv->dialog_up) {
+                gs_debug ("Request unlock but dialog is already up");
+                return FALSE;
         }
 
         if (manager->priv->fading) {
@@ -1272,19 +1412,14 @@ gs_manager_request_unlock (GSManager *manager)
                 gs_fade_finish (manager->priv->fade);
         }
 
-        /* Find the screen that contains the pointer */
-        display = gdk_display_get_default ();
-        gdk_display_get_pointer (display, &screen, NULL, NULL, NULL);
-        screen_num = gdk_screen_get_number (screen);
-
-        /* Find the gs-window that is on that screen */
-        for (l = manager->priv->windows; l; l = l->next) {
-                int num;
-
-                num = gdk_screen_get_number (GTK_WINDOW (l->data)->screen);
-                if (num == screen_num) {
-                        gs_window_request_unlock (GS_WINDOW (l->data));
-                        break;
-                }
+        if (manager->priv->windows == NULL) {
+                gs_debug ("We don't have any windows!");
+                return FALSE;
         }
+
+        /* Find the GSWindow that contains the pointer */
+        window = find_window_at_pointer (manager);
+        gs_window_request_unlock (window);
+
+        return TRUE;
 }
