@@ -47,7 +47,7 @@ typedef struct
         GObjectClass     parent_class;
 
         void            (* activated)       (GSManager *manager);
-        void            (* disactivated)    (GSManager *manager);
+        void            (* deactivated)     (GSManager *manager);
 
 } GSManagerClass;
 
@@ -57,11 +57,14 @@ GSManager * gs_manager_new                  (void);
 
 gboolean    gs_manager_set_active           (GSManager  *manager,
                                              gboolean    active);
+gboolean    gs_manager_get_active           (GSManager  *manager);
+
 gboolean    gs_manager_cycle                (GSManager  *manager);
 
-gboolean    gs_manager_is_active            (GSManager  *manager);
+void        gs_manager_get_lock_active      (GSManager  *manager,
+                                             gboolean   *lock_active);
 void        gs_manager_set_lock_active      (GSManager  *manager,
-                                             gboolean    lock_enabled);
+                                             gboolean    lock_active);
 void        gs_manager_set_lock_enabled     (GSManager  *manager,
                                              gboolean    lock_enabled);
 void        gs_manager_set_lock_timeout     (GSManager  *manager,
@@ -82,7 +85,7 @@ void        gs_manager_set_themes           (GSManager  *manager,
                                              GSList     *themes);
 void        gs_manager_set_mode             (GSManager  *manager,
                                              GSSaverMode mode);
-void        gs_manager_request_unlock       (GSManager  *manager);
+gboolean    gs_manager_request_unlock       (GSManager  *manager);
 
 G_END_DECLS
 
