@@ -32,6 +32,7 @@
 #include <gtk/gtk.h>
 
 #include "gs-window.h"
+#include "gs-marshal.h"
 #include "subprocs.h"
 #include "gs-debug.h"
 
@@ -40,7 +41,6 @@ static void gs_window_init       (GSWindow      *window);
 static void gs_window_finalize   (GObject       *object);
 
 static gboolean popup_dialog_idle (GSWindow *window);
-static gboolean gs_window_request_unlock_idle (GSWindow *window);
 
 enum {
         DIALOG_RESPONSE_CANCEL,
@@ -54,6 +54,7 @@ struct GSWindowPrivate
         int        monitor;
 
         GdkRectangle geometry;
+        guint      obscured : 1;
 
         guint      lock_enabled : 1;
         guint      user_switch_enabled : 1;
@@ -64,7 +65,6 @@ struct GSWindowPrivate
         GtkWidget *box;
         GtkWidget *socket;
 
-        guint      request_unlock_idle_id;
         guint      popup_dialog_idle_id;
 
         guint      dialog_map_signal_id;
@@ -86,6 +86,7 @@ struct GSWindowPrivate
 };
 
 enum {
+        ACTIVITY,
         DEACTIVATED,
         DIALOG_UP,
         DIALOG_DOWN,
@@ -94,6 +95,7 @@ enum {
 
 enum {
         PROP_0,
+        PROP_OBSCURED,
         PROP_LOCK_ENABLED,
         PROP_LOGOUT_ENABLED,
         PROP_LOGOUT_COMMAND,
@@ -161,6 +163,32 @@ gs_window_override_user_time (GSWindow *window)
         gdk_x11_window_set_user_time (GTK_WIDGET (window)->window, ev_time);
 }
 
+static void
+force_no_pixmap_background (GtkWidget *widget)
+{
+        static gboolean first_time = TRUE;
+
+        if (first_time) {
+                gtk_rc_parse_string ("\n"
+                                     "   style \"gs-theme-engine-style\"\n"
+                                     "   {\n"
+                                     "      bg_pixmap[NORMAL] = \"<none>\"\n"
+                                     "      bg_pixmap[INSENSITIVE] = \"<none>\"\n"
+                                     "      bg_pixmap[ACTIVE] = \"<none>\"\n"
+                                     "      bg_pixmap[PRELIGHT] = \"<none>\"\n"
+                                     "      bg[NORMAL] = { 0.0, 0.0, 0.0 }\n"
+                                     "      bg[INSENSITIVE] = { 0.0, 0.0, 0.0 }\n"
+                                     "      bg[ACTIVE] = { 0.0, 0.0, 0.0 }\n"
+                                     "      bg[PRELIGHT] = { 0.0, 0.0, 0.0 }\n"
+                                     "   }\n"
+                                     "   widget \"gs-window*\" style : highest \"gs-theme-engine-style\"\n"
+                                     "\n");
+                first_time = FALSE;
+        }
+
+        gtk_widget_set_name (widget, "gs-window");
+}
+
 static void
 clear_children (Window window)
 {
@@ -327,7 +355,9 @@ gs_window_real_realize (GtkWidget *widget)
 static gboolean
 watchdog_timer (GSWindow *window)
 {
-        gtk_window_present (GTK_WINDOW (window));
+        GtkWidget *widget = GTK_WIDGET (window);
+
+        gdk_window_focus (widget->window, GDK_CURRENT_TIME);
 
         return TRUE;
 }
@@ -365,21 +395,6 @@ add_popup_dialog_idle (GSWindow *window)
         window->priv->popup_dialog_idle_id = g_idle_add ((GSourceFunc)popup_dialog_idle, window);
 }
 
-static void
-remove_request_unlock_idle (GSWindow *window)
-{
-        if (window->priv->request_unlock_idle_id != 0) {
-                g_source_remove (window->priv->request_unlock_idle_id);
-                window->priv->request_unlock_idle_id = 0;
-        }
-}
-
-static void
-add_request_unlock_idle (GSWindow *window)
-{
-        window->priv->request_unlock_idle_id = g_idle_add ((GSourceFunc)gs_window_request_unlock_idle, window);
-}
-
 static gboolean
 emit_deactivated_idle (GSWindow *window)
 {
@@ -1186,30 +1201,24 @@ gs_window_get_property (GObject    *object,
         case PROP_MONITOR:
                 g_value_set_int (value, self->priv->monitor);
                 break;
+        case PROP_OBSCURED:
+                g_value_set_boolean (value, self->priv->obscured);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
         }
 }
 
-static gboolean
-gs_window_request_unlock_idle (GSWindow *window)
-{
-        gs_window_request_unlock (window);
-
-        window->priv->request_unlock_idle_id = 0;
-
-        return FALSE;
-}
-
 static void
 queue_key_event (GSWindow    *window,
                  GdkEventKey *event)
 {
-        /* Eat the first return, enter, or space */
+        /* Eat the first return, enter, escape, or space */
         if (window->priv->key_events == NULL
             && (event->keyval == GDK_Return
                 || event->keyval == GDK_KP_Enter
+                || event->keyval == GDK_Escape
                 || event->keyval == GDK_space)) {
                 return;
         }
@@ -1219,20 +1228,19 @@ queue_key_event (GSWindow    *window,
 }
 
 static gboolean
-maybe_request_unlock (GSWindow *window)
+maybe_handle_activity (GSWindow *window)
 {
-        gboolean ret;
+        gboolean handled;
 
-        ret = FALSE;
+        handled = FALSE;
 
-        /* if we don't already have a socket then request an unlock */
+        /* if we already have a socket then don't bother */
         if (! window->priv->socket
-            && (window->priv->request_unlock_idle_id == 0)) {
-                add_request_unlock_idle (window);
-                ret = TRUE;
+            && GTK_WIDGET_IS_SENSITIVE (GTK_WIDGET (window))) {
+                g_signal_emit (window, signals [ACTIVITY], 0, &handled);
         }
 
-        return ret;
+        return handled;
 }
 
 static gboolean
@@ -1243,16 +1251,7 @@ gs_window_real_key_press_event (GtkWidget   *widget,
 
         /*g_message ("KEY PRESS state: %u keyval %u", event->state, event->keyval);*/
 
-        /* if we don't already have a socket then request an unlock */
-        if (! GS_WINDOW (widget)->priv->socket) {
-                maybe_request_unlock (GS_WINDOW (widget));
-
-                catch_events = TRUE;
-        } else {
-                if (! GTK_WIDGET_VISIBLE (GS_WINDOW (widget)->priv->socket)) {
-                        catch_events = TRUE;
-                }
-        }
+        catch_events = maybe_handle_activity (GS_WINDOW (widget));
 
         /* Catch all keypresses up until the lock dialog is shown */
         if (catch_events) {
@@ -1289,7 +1288,7 @@ gs_window_real_motion_notify_event (GtkWidget      *widget,
                         ABS (window->priv->last_y - event->y));
 
         if (distance > min_distance) {
-                maybe_request_unlock (window);
+                maybe_handle_activity (window);
 
                 window->priv->last_x = -1;
                 window->priv->last_y = -1;
@@ -1305,7 +1304,7 @@ gs_window_real_button_press_event (GtkWidget      *widget,
         GSWindow *window;
 
         window = GS_WINDOW (widget);
-        maybe_request_unlock (window);
+        maybe_handle_activity (window);
 
         return FALSE;
 }
@@ -1317,7 +1316,7 @@ gs_window_real_scroll_event (GtkWidget      *widget,
         GSWindow *window;
 
         window = GS_WINDOW (widget);
-        maybe_request_unlock (window);
+        maybe_handle_activity (window);
 
         return FALSE;
 }
@@ -1381,6 +1380,46 @@ gs_window_real_grab_broken (GtkWidget          *widget,
         return FALSE;
 }
 
+gboolean
+gs_window_is_obscured (GSWindow *window)
+{
+        g_return_val_if_fail (GS_IS_WINDOW (window), FALSE);
+
+        return window->priv->obscured;
+}
+
+static void
+window_set_obscured (GSWindow *window,
+                     gboolean  obscured)
+{
+        if (window->priv->obscured == obscured) {
+                return;
+        }
+
+        window->priv->obscured = obscured;
+        g_object_notify (G_OBJECT (window), "obscured");
+}
+
+static gboolean
+gs_window_real_visibility_notify_event (GtkWidget          *widget,
+                                        GdkEventVisibility *event)
+{
+        switch (event->state) {
+        case GDK_VISIBILITY_FULLY_OBSCURED:
+                window_set_obscured (GS_WINDOW (widget), TRUE);
+                break;
+        case GDK_VISIBILITY_PARTIAL:
+                break;
+        case GDK_VISIBILITY_UNOBSCURED:
+                window_set_obscured (GS_WINDOW (widget), FALSE);
+                break;
+        default:
+                break;
+        }
+
+        return FALSE;
+}
+
 static void
 gs_window_class_init (GSWindowClass *klass)
 {
@@ -1403,9 +1442,20 @@ gs_window_class_init (GSWindowClass *klass)
         widget_class->scroll_event        = gs_window_real_scroll_event;
         widget_class->size_request        = gs_window_real_size_request;
         widget_class->grab_broken_event   = gs_window_real_grab_broken;
+        widget_class->visibility_notify_event = gs_window_real_visibility_notify_event;
 
         g_type_class_add_private (klass, sizeof (GSWindowPrivate));
 
+        signals [ACTIVITY] =
+                g_signal_new ("activity",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GSWindowClass, activity),
+                              NULL,
+                              NULL,
+                              gs_marshal_BOOLEAN__VOID,
+                              G_TYPE_BOOLEAN,
+                              0);
         signals [DEACTIVATED] =
                 g_signal_new ("deactivated",
                               G_TYPE_FROM_CLASS (object_class),
@@ -1437,6 +1487,13 @@ gs_window_class_init (GSWindowClass *klass)
                               G_TYPE_NONE,
                               0);
 
+        g_object_class_install_property (object_class,
+                                         PROP_OBSCURED,
+                                         g_param_spec_boolean ("obscured",
+                                                               NULL,
+                                                               NULL,
+                                                               FALSE,
+                                                               G_PARAM_READABLE));
         g_object_class_install_property (object_class,
                                          PROP_LOCK_ENABLED,
                                          g_param_spec_boolean ("lock-enabled",
@@ -1461,7 +1518,7 @@ gs_window_class_init (GSWindowClass *klass)
                                                             0,
                                                             G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
-                                         PROP_LOGOUT_TIMEOUT,
+                                         PROP_LOGOUT_COMMAND,
                                          g_param_spec_string ("logout-command",
                                                               NULL,
                                                               NULL,
@@ -1510,8 +1567,10 @@ gs_window_init (GSWindow *window)
                                | GDK_KEY_PRESS_MASK
                                | GDK_KEY_RELEASE_MASK
                                | GDK_EXPOSURE_MASK
+                               | GDK_VISIBILITY_NOTIFY_MASK
                                | GDK_ENTER_NOTIFY_MASK
                                | GDK_LEAVE_NOTIFY_MASK);
+        force_no_pixmap_background (GTK_WIDGET (window));
 }
 
 static void
@@ -1538,7 +1597,6 @@ gs_window_finalize (GObject *object)
         g_free (window->priv->logout_command);
 
         remove_watchdog_timer (window);
-        remove_request_unlock_idle (window);
         remove_popup_dialog_idle (window);
 
         if (window->priv->timer) {
